﻿using System.Windows;
using System;
using System.Threading;
using System.Threading.Tasks;
using Telegram.Bot;
using Telegram.Bot.Extensions.Polling;
using Telegram.Bot.Types;
using RestSharp;
using RestSharp.Authenticators;
using System.Text.Json;
using System.Collections.Generic;

namespace Telegram
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        static ITelegramBotClient bot;
        public string Domain = "", Login = "", Password = "";

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Domain = Apipath.Text;
            Login = login.Text;
            Password = password.Password;

            bot = new TelegramBotClient(apitoken.Text);
            var cts = new CancellationTokenSource();
            var cancellationToken = cts.Token;
            var receiverOptions = new ReceiverOptions
            {
                AllowedUpdates = { } // receive all update types
            };
            bot.StartReceiving(
                HandleUpdateAsync,
                HandleErrorAsync,
                receiverOptions,
                cancellationToken
            );
            MessageBox.Show("Бот запущен");
        }

        public async Task HandleUpdateAsync(ITelegramBotClient botClient, Update update, CancellationToken cancellationToken)
        {
            try
            {
                if (update.Type == Telegram.Bot.Types.Enums.UpdateType.Message)
                {
                    var message = update.Message;

                    string data = Get("Sotrudniki");
                    if (ValueFromJSON(data, "ChatID", message.Chat.Id.ToString()) == "")
                    {
                        if (message.Text.Contains("@"))
                        {
                            var doc = JsonDocument.Parse(data);
                            JsonElement root = doc.RootElement;
                            var users = root.EnumerateArray();
                            while (users.MoveNext())
                            {
                                var user = users.Current;
                                var props = user.EnumerateObject();
                                string account_id = "", current_email = "";
                                while (props.MoveNext())
                                {
                                    var prop = props.Current;
                                    if (prop.Name.ToLower() == "mbox")
                                    {
                                        current_email = prop.Value.ToString();
                                    }
                                    if (prop.Name.ToLower() == "id")
                                    {
                                        account_id = prop.Value.ToString();
                                    }
                                }
                                if (current_email.ToLower() == message.Text.ToLower())
                                {
                                    var symbols = "0123456789";
                                    string code = "";
                                    var random = new Random();
                                    for (int i = 0; i < 4; i++)
                                    {
                                        code += symbols[random.Next(symbols.Length)].ToString();
                                    }
                                    // найденному пользователю в платформе проставим ChatID с пометкой _NA (т.е. не авторизован)
                                    // и сгенерированный 4-х значный код
                                    var data_ = new Dictionary<string, object>
                                        {
                                            {"op.13", message.Chat.Id.ToString() + "_NA"},//ChatId
                                            {"op.14", code}//Proverochnyykod
                                        };
                                    var client = new RestClient(Domain);
                                    client.Authenticator = new HttpBasicAuthenticator(Login, Password);
                                    var request = new RestSharp.RestRequest("system/TeamNetwork/ObjectService/Edit", Method.Post); // POST-запрос к найденному пользователю для обновления данных
                                    request.AddHeader("content-type", "application/json");
                                    var sendData = new Dictionary<string, object>
                                    {
                                        {"objectId", account_id},
                                        {"objectData", data_ }
                                    };
                                    request.AddParameter("application/json", JsonSerializer.Serialize(sendData), ParameterType.RequestBody);
                                    client.Execute(request);

                                    
                                    // запустим процесс отправки сгенерированного кода на email
                                    var data_for_proccess = new Dictionary<string, object>
                                        {
                                            {"op.23", code}, // Код
                                            {"op.24", current_email} // Кому
                                        };
                                    request = new RestSharp.RestRequest("system/Process/ProcessObjectService/Create1", Method.Post);
                                    request.AddHeader("content-type", "application/json");
                                    data_ = new Dictionary<string, object>
                                        {
                                            {"processAppId", "pa.3"}, // process_id
                                            {"objectName", null},
                                            {"syncActivityQuantity", 2},
                                            {"objectData", data_for_proccess}
                                        };
                                    request.AddParameter("application/json", JsonSerializer.Serialize(data_), ParameterType.RequestBody);
                                    client.Execute(request);
                                    
                                    await bot.SendTextMessageAsync(message.Chat.Id, "На указанную почту отправлено письмо с 4-х значным кодом, введите его ниже для авторизации");
                                }
                            }
                        }
                        else if (message.Text.Length == 4)
                        {
                            string value = ValueFromJSON(data, "Proverochnyykod", message.Chat.Id.ToString() + "_NA");
                            if (message.Text == value)
                            {
                                // у ChatID убираем пометку "_NA" и очищаем поле "Проверочный код"
                                var data_ = new Dictionary<string, object>
                                    {
                                        {"op.13", message.Chat.Id.ToString()},//ChatId
                                        {"op.14", null}//Proverochnyykod
                                    };
                                var client = new RestClient(Domain);
                                client.Authenticator = new HttpBasicAuthenticator(Login, Password);
                                value = ValueFromJSON(data, "id", message.Chat.Id.ToString() + "_NA");
                                var request = new RestSharp.RestRequest("system/TeamNetwork/ObjectService/Edit", Method.Post);
                                request.AddHeader("content-type", "application/json");
                                var sendData = new Dictionary<string, object>
                                {
                                    {"objectId", value},
                                    {"objectData", data_}
                                };
                                request.AddParameter("application/json", JsonSerializer.Serialize(sendData), ParameterType.RequestBody);
                                client.Execute(request);
                                await bot.SendTextMessageAsync(message.Chat.Id, "Вы успешно авторизованы");
                            }
                        }
                        else
                        {
                            await bot.SendTextMessageAsync(message.Chat.Id, "Укажите email-адрес для авторизации");
                        }
                    }
                    else
                    {
                        string name = ValueFromJSON(data, "fullName", message.Chat.Id.ToString());
                        await bot.SendTextMessageAsync(message.Chat.Id, "Добрый день, " + name + "! Вы успешно авторизованы");
                    }
                }
            }
            catch { }
        }

        public async Task HandleErrorAsync(ITelegramBotClient botClient, Exception exception, CancellationToken cancellationToken)
        {
            MessageBox.Show("Ошибка " + exception.Message);
        }

        private string Get(string OA_system_name)
        {
            var client = new RestClient(Domain);
            client.Authenticator = new HttpBasicAuthenticator(Login, Password);
            var request = new RestRequest("solution/" + OA_system_name, Method.Get);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute(request);
            return response.Content;
        }

        private string ValueFromJSON(string data, string attribute_name, string chat_id)
        {
            var doc = JsonDocument.Parse(data);
            JsonElement root = doc.RootElement;
            var users = root.EnumerateArray();
            while (users.MoveNext())
            {
                var user = users.Current;
                var props = user.EnumerateObject();
                string value_ = "", chat_id_ = "";
                while (props.MoveNext())
                {
                    var prop = props.Current;
                    if (prop.Name.ToLower() == attribute_name.ToLower())
                    {
                        value_ = prop.Value.ToString();
                    }
                    if (prop.Name.ToLower() == "chatid")
                    {
                        chat_id_ = prop.Value.ToString();
                    }
                }
                if (chat_id_ == chat_id)
                {
                    return value_;
                }
            }
            return "";
        }
    }
}
